#!/bin/env python
#copyright ReportLab Inc. 2000
#see license.txt for license details
#history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/lib/__init__.py?cvsroot=reportlab
#$Header: /cvsroot/reportlab/reportlab/lib/__init__.py,v 1.4 2002/07/24 19:56:37 andy_robinson Exp $
__version__=''' $Id: __init__.py,v 1.4 2002/07/24 19:56:37 andy_robinson Exp $ '''
import os
RL_DEBUG = os.environ.has_key('RL_DEBUG')