#copyright ReportLab Inc. 2000-2002
#see license.txt for license details
#history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/tools/docco/__init__.py?cvsroot=reportlab
#$Header: /cvsroot/reportlab/reportlab/tools/docco/__init__.py,v 1.3 2002/07/24 19:56:39 andy_robinson Exp $