#copyright ReportLab Inc. 2000
#see license.txt for license details
#history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/lib/enums.py?cvsroot=reportlab
#$Header: /cvsroot/reportlab/reportlab/lib/enums.py,v 1.3 2002/07/24 19:56:37 andy_robinson Exp $
__version__=''' $Id: enums.py,v 1.3 2002/07/24 19:56:37 andy_robinson Exp $ '''
__doc__="""
holder for all reportlab's enumerated types
"""
TA_LEFT = 0
TA_CENTER = 1
TA_RIGHT = 2
TA_JUSTIFY = 4