#copyright ReportLab Inc. 2000
#see license.txt for license details
#history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/extensions/__init__.py?cvsroot=reportlab
#$Header: /cvsroot/reportlab/reportlab/extensions/__init__.py,v 1.3 2002/07/24 19:56:36 andy_robinson Exp $
__version__=''' $Id: __init__.py,v 1.3 2002/07/24 19:56:36 andy_robinson Exp $ '''
__doc__="""
"""
# No usable content. Do not add to this file!
